@@ -1,15 +1,16 @@
 #[cfg(test)]
 mod tests_double_map;
 
-use core::hash::{BuildHasher, Hash};
 use core::borrow::Borrow;
+use core::default::Default;
+use core::fmt::{self, Debug};
+use core::hash::{BuildHasher, Hash};
+use core::hint::unreachable_unchecked;
+use core::iter::{Extend, FromIterator};
+use core::mem;
 use std::collections::hash_map;
 use std::collections::HashMap;
 use std::collections::TryReserveError;
-use core::fmt::{self, Debug};
-use core::hint::unreachable_unchecked;
-use core::mem;
-use core::iter::{FromIterator, Extend};
 
 /// A hash map with double keys implemented as wrapper above two
 /// [`HashMaps`](`std::collections::HashMap`).
@@ -293,7 +294,7 @@ impl<K1, K2, V, S> DHashMap<K1, K2, V, S> {
     /// # Examples
     ///
     /// ```
-    /// use double_map::DHashMap;
+    /// use double_map::{DHashMap, dhashmap};
     ///
     /// let mut a = DHashMap::new();
     /// // The created DHashMap doesn't hold any elements
@@ -303,9 +304,10 @@ impl<K1, K2, V, S> DHashMap<K1, K2, V, S> {
     /// // And can be sure that DHashMap holds one element
     /// assert_eq!(a.len(), 1);
     ///
-    /// let mut map = DHashMap::new();
-    /// map.insert(1, "Breakfast", "Pancakes");
-    /// map.insert(2, "Lunch", "Sandwich");
+    /// let map = dhashmap![
+    ///    1, "Breakfast" => "Pancakes",
+    ///    2, "Lunch" => "Sandwich",
+    /// ];
     /// assert_eq!(map.len(), 2);
     /// ```
     #[inline]
@@ -343,11 +345,12 @@ impl<K1, K2, V, S> DHashMap<K1, K2, V, S> {
     /// # Examples
     ///
     /// ```
-    /// use double_map::DHashMap;
+    /// use double_map::{DHashMap, dhashmap};
     ///
-    /// let mut a = DHashMap::new();
-    /// a.insert(1, "Breakfast", "Pancakes");
-    /// a.insert(2, "Lunch", "Sandwich");
+    /// let mut a = dhashmap![
+    ///    1, "Breakfast" => "Pancakes",
+    ///    2, "Lunch" => "Sandwich",
+    /// ];
     ///
     /// // We can that see DHashMap holds two elements
     /// assert_eq!(a.len(), 2);
@@ -656,13 +659,14 @@ where
     /// # Examples
     ///
     /// ```
-    /// use double_map::DHashMap;
+    /// use double_map::{DHashMap, dhashmap};
     ///
     /// // We create map with three elements
-    /// let mut map = DHashMap::new();
-    /// map.insert(1, "One", String::from("Eins"));
-    /// map.insert(2, "Two", String::from("Zwei"));
-    /// map.insert(3, "Three", String::from("Drei"));
+    /// let mut map = dhashmap! {
+    ///     1, "One"   => String::from("Eins"),
+    ///     2, "Two"   => String::from("Zwei"),
+    ///     3, "Three" => String::from("Drei"),
+    /// };
     ///
     /// // We can see that DHashMap holds three elements
     /// assert!(map.len() == 3 && map.capacity() >= 3);
@@ -713,13 +717,14 @@ where
     /// # Examples
     ///
     /// ```
-    /// use double_map::DHashMap;
+    /// use double_map::{DHashMap, dhashmap};
     ///
     /// // We create map with three elements
-    /// let mut map = DHashMap::new();
-    /// map.insert(1, "One", String::from("Eins"));
-    /// map.insert(2, "Two", String::from("Zwei"));
-    /// map.insert(3, "Three", String::from("Drei"));
+    /// let mut map = dhashmap! {
+    ///     1, "One"   => String::from("Eins"),
+    ///     2, "Two"   => String::from("Zwei"),
+    ///     3, "Three" => String::from("Drei"),
+    /// };
     ///
     /// // We can see that DHashMap holds three elements
     /// assert!(map.len() == 3 && map.capacity() >= 3);
@@ -830,7 +835,7 @@ where
             None => match self.key_map.get(&k2) {
                 None => {
                     // SAFETY: We already check that both key vacant
-                    Ok( unsafe { self.map_vacant_entry(k1, k2) } )
+                    Ok(unsafe { self.map_vacant_entry(k1, k2) })
                 }
                 // Error: Vacant key #1 of type K1 and occupied key # 2 of type K2
                 Some(_) => Err(EntryError {
@@ -842,7 +847,7 @@ where
                 Some(key1_exist) => {
                     return if k1 == *key1_exist && k2 == *key2_exist {
                         // SAFETY: We already check that both key exist and refer to the same value
-                        Ok( unsafe { self.map_occupied_entry(k1, k2) } )
+                        Ok(unsafe { self.map_occupied_entry(k1, k2) })
                     } else {
                         // Error: key #1 and key # 2 refer to different entries / values
                         Err(EntryError {
@@ -884,9 +889,7 @@ where
         let raw_k = self.key_map.entry(k2);
         match raw_v {
             hash_map::Entry::Vacant(base_v) => match raw_k {
-                hash_map::Entry::Vacant(base_k) => {
-                    Entry::Vacant(VacantEntry { base_v, base_k })
-                },
+                hash_map::Entry::Vacant(base_k) => Entry::Vacant(VacantEntry { base_v, base_k }),
                 _ => unreachable_unchecked(),
             },
             _ => unreachable_unchecked(),
@@ -1209,12 +1212,127 @@ where
     }
 }
 
+/// Create a `DHashMap<K1, K2, V, `[`RandomState`](std::collections::hash_map::RandomState)`>`
+/// from a list of sequentially given keys and values.
+///
+/// Input data list must follow one of these rules:
+/// - `K1, K2 => V, K1, K2 => V` ... and so on;
+/// - `(K1, K2) => V, (K1, K2) => V` ... and so on.
+///
+/// Last comma separator can be omitted.
+/// If this macros is called without arguments, i.e. like
+/// ```
+/// # use double_map::{DHashMap, dhashmap};
+/// let map: DHashMap<i32, String, String>  = dhashmap![];
+/// ```
+/// it is equivalent to [`DHashMap::new()`] function
+///
+/// # Examples
+///
+/// ```
+/// use double_map::{DHashMap, dhashmap};
+///
+/// // Calling macros without arguments is equivalent to DHashMap::new() function
+/// let _map0: DHashMap<i32, i32, i32> = dhashmap![];
+///
+/// let map = dhashmap!{
+///     1, "a" => "One",
+///     2, "b" => "Two", // last comma separator can be omitted
+/// };
+///
+/// assert_eq!(map.get_key1(&1),   Some(&"One"));
+/// assert_eq!(map.get_key1(&2),   Some(&"Two"));
+/// assert_eq!(map.get_key2(&"a"), Some(&"One"));
+/// assert_eq!(map.get_key2(&"b"), Some(&"Two"));
+///
+/// let map2 = dhashmap!{
+///     (3, "c") => "Three",
+///     (4, "d") => "Four" // last comma separator can be omitted
+/// };
+///
+/// assert_eq!(map2.get_key1(&3),   Some(&"Three"));
+/// assert_eq!(map2.get_key1(&4),   Some(&"Four"));
+/// assert_eq!(map2.get_key2(&"c"), Some(&"Three"));
+/// assert_eq!(map2.get_key2(&"d"), Some(&"Four"));
+/// ```
+#[macro_export]
+macro_rules! dhashmap {
+    () => (DHashMap::new());
+    ($($key1:expr, $key2:expr => $value:expr),+ $(,)?) => (
+        DHashMap::<_, _, _, std::collections::hash_map::RandomState>::from_iter([$(($key1, $key2, $value)),+])
+    );
+    ($(($key1:expr, $key2:expr) => $value:expr),+ $(,)?) => (
+        DHashMap::<_, _, _, std::collections::hash_map::RandomState>::from_iter([$(($key1, $key2, $value)),+])
+    );
+}
+
+/// Creates an empty `DHashMap<K1, K2, V, S>`, with the `Default` value for the hasher.
+impl<K1, K2, V, S> Default for DHashMap<K1, K2, V, S>
+where
+    S: Default + Clone,
+{
+    /// Creates an empty `DHashMap<K1, K2, V, S>`, with the `Default` value for the hasher.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use double_map::DHashMap;
+    /// use std::collections::hash_map::RandomState;
+    ///
+    /// // You need to specify all types of DHashMap, including hasher.
+    /// // Created map is empty and don't allocate memory
+    /// let map: DHashMap<u32, String, String, RandomState> = Default::default();
+    /// assert_eq!(map.capacity(), 0);
+    /// let map: DHashMap<u32, String, String, RandomState> = DHashMap::default();
+    /// assert_eq!(map.capacity(), 0);
+    /// ```
+    #[inline]
+    fn default() -> DHashMap<K1, K2, V, S> {
+        DHashMap::with_hasher(Default::default())
+    }
+}
+
+/// Creates an new `DHashMap<K1, K2, V, S>`, with the `Default` value
+/// for the hasher from from an iterator.
 impl<K1, K2, V, S> FromIterator<(K1, K2, V)> for DHashMap<K1, K2, V, S>
 where
     K1: Eq + Hash + Clone,
     K2: Eq + Hash + Clone,
     S: BuildHasher + Default + Clone,
 {
+    /// Creates an new `DHashMap<K1, K2, V, S>`, with the `Default` value
+    /// for the hasher from from an iterator.
+    ///
+    /// You need to specify the type of `hasher`
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use double_map::DHashMap;
+    /// use std::collections::hash_map::RandomState;
+    ///
+    /// let mut number = 0;
+    /// let some_iter = std::iter::repeat_with(move || {
+    ///     number +=1;
+    ///     (number, number, number * 10)
+    /// }).take(5);
+    /// // You need to specify hasher
+    /// let map: DHashMap<_, _, _, RandomState> = DHashMap::from_iter(some_iter.clone());
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    /// assert_eq!(map.get_key1(&5), Some(&50));
+    /// assert_eq!(map.get_key1(&6), None);
+    ///
+    /// let some_vec: Vec<_> = some_iter.collect();
+    /// let map: DHashMap<_, _, _, RandomState> = DHashMap::from_iter(some_vec);
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    /// assert_eq!(map.get_key1(&5), Some(&50));
+    /// assert_eq!(map.get_key1(&6), None);
+    ///
+    /// let some_arr = [(1, 1, 10), (2, 2, 20), (3, 3, 30), (4, 4, 40), (5, 5, 50)];
+    /// let map: DHashMap<_, _, _, RandomState> = DHashMap::from_iter(some_arr);
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    /// assert_eq!(map.get_key1(&5), Some(&50));
+    /// assert_eq!(map.get_key1(&6), None);
     fn from_iter<T: IntoIterator<Item = (K1, K2, V)>>(iter: T) -> DHashMap<K1, K2, V, S> {
         let mut map = DHashMap::with_hasher(Default::default());
         map.extend(iter);
@@ -1222,14 +1340,61 @@ where
     }
 }
 
-/// Inserts all new keys and values from the iterator. Replace values with existing
-/// keys with new values returned from the iterator.
+/// Inserts all new keys and values from the iterator to existing `DHashMap<K1, K2, V, S>`.
 impl<K1, K2, V, S> Extend<(K1, K2, V)> for DHashMap<K1, K2, V, S>
 where
     K1: Eq + Hash + Clone,
     K2: Eq + Hash + Clone,
     S: BuildHasher,
 {
+    /// Inserts all new keys and values from the iterator to existing `DHashMap<K1, K2, V, S>`.
+    ///
+    /// Replace values with existing keys with new values returned from the iterator.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use double_map::DHashMap;
+    ///
+    /// // Let's create `DHashMap` with std::collections::hash_map::RandomState hasher
+    /// let mut map = DHashMap::new();
+    /// map.insert(1, 1, 999);
+    ///
+    /// let mut number = 0;
+    /// let some_iter = std::iter::repeat_with(move || {
+    ///     number +=1;
+    ///     (number, number, number * 10)
+    /// }).take(5);
+    ///
+    /// // You don't need to specify the hasher
+    /// map.extend(some_iter);
+    /// // Replace values with existing keys with new values returned from the iterator.
+    /// // So that the map.get_key1(&1) doesn't return Some(&999).
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    ///
+    /// let some_vec: Vec<_> = std::iter::repeat_with(move || {
+    ///     number +=100;
+    ///     (number, number, number * 10)
+    /// }).take(5).collect();
+    /// map.extend(some_vec);
+    ///
+    /// let some_arr = [(11, 11, 111), (22, 22, 222), (33, 33, 333), (44, 44, 4444), (55, 55, 555)];
+    /// map.extend(some_arr);
+    ///
+    /// // Keys and values from some_iter
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    /// assert_eq!(map.get_key1(&5), Some(&50));
+    /// assert_eq!(map.get_key1(&6), None);
+    ///
+    /// // Keys and values from some_vec
+    /// assert_eq!(map.get_key1(&100), Some(&1000));
+    /// assert_eq!(map.get_key1(&500), Some(&5000));
+    /// assert_eq!(map.get_key1(&600), None);
+    ///
+    /// // Keys and values from some_arr
+    /// assert_eq!(map.get_key1(&11), Some(&111));
+    /// assert_eq!(map.get_key1(&55), Some(&555));
+    /// assert_eq!(map.get_key1(&66), None);
     #[inline]
     fn extend<T: IntoIterator<Item = (K1, K2, V)>>(&mut self, iter: T) {
         // Keys may be already present or show multiple times in the iterator.
@@ -1249,9 +1414,7 @@ where
     }
 }
 
-/// Inserts all new keys and values from the iterator. Replace values with existing
-/// keys with new values returned from the iterator. The keys and values must
-/// implement [`Copy`] trait.
+/// Inserts all new keys and values from the iterator to existing `DHashMap<K1, K2, V, S>`.
 impl<'a, K1, K2, V, S> Extend<(&'a K1, &'a K2, &'a V)> for DHashMap<K1, K2, V, S>
 where
     K1: Eq + Hash + Copy,
@@ -1259,6 +1422,38 @@ where
     V: Copy,
     S: BuildHasher,
 {
+    /// Inserts all new keys and values from the iterator to existing `DHashMap<K1, K2, V, S>`.
+    ///
+    /// Replace values with existing keys with new values returned from the iterator.
+    /// The keys and values must implement [`Copy`] trait.
+    ///
+    /// # Examples
+    ///
+    /// ```
+    /// use double_map::DHashMap;
+    ///
+    /// // Let's create `DHashMap` with std::collections::hash_map::RandomState hasher
+    /// let mut map = DHashMap::new();
+    /// map.insert(1, 1, 999);
+    ///
+    /// let mut number = 0;
+    /// let some_vec: Vec<_> = std::iter::repeat_with(move || {
+    ///     number +=1;
+    ///     (number, number, number * 10)
+    /// }).take(5).collect();
+    ///
+    /// // You don't need to specify the hasher
+    /// let some_iter = some_vec.iter().map(|(k1, k2, v)| (k1, k2, v));
+    /// map.extend(some_iter);
+    ///
+    /// // Replace values with existing keys with new values returned from the iterator.
+    /// // So that the map.get_key1(&1) doesn't return Some(&999).
+    /// assert_eq!(map.get_key1(&1), Some(&10));
+    /// assert_eq!(map.get_key1(&5), Some(&50));
+    /// assert_eq!(map.get_key1(&6), None);
+    ///
+    /// // And created vector are still can be used.
+    /// assert_eq!(some_vec[4], (5, 5, 50));
     #[inline]
     fn extend<T: IntoIterator<Item = (&'a K1, &'a K2, &'a V)>>(&mut self, iter: T) {
         self.extend(iter.into_iter().map(|(&k1, &k2, &v)| (k1, k2, v)))
@@ -2355,4 +2550,4 @@ where
     K2: Eq + Hash + Clone + Debug,
     V: Debug,
 {
-}
\ No newline at end of file
+}
